// balmer@inbox.ru 2013 TermometerHygrometer
#include "command.h"
#include "dht.h"
#include "main.h"
#include "ti8077_lcd.h"
#include "stm32l1xx_flash.h"

void RtcReadTimeToStr(void);

void ReadFlashData32(uint32_t address)
{
	int i;
	if(address>=FLASH_STORAGE_SIZE)
	{
		USBAdd8(0);
		return;
	}

	address+=FLASH_START_ARRAY;

	for(i=0; i<8; i++)
		USBAdd32(*(i+(uint32_t*)address));
}

void WriteFlashData32(uint32_t address, uint8_t* data)
{
	int i;
	FLASH_Status status;
	if(address>=FLASH_STORAGE_SIZE)
	{
		USBAdd8(0);
		return;
	}

	address+=FLASH_START_ARRAY;
	FLASH_Unlock();
	for(i=0; i<32; i+=4)
	{
		status = FLASH_FastProgramWord(address+i, *(uint32_t*)(data+i));
		if(status!=FLASH_COMPLETE)
			break;
	}
	FLASH_Lock();
	USBAdd8(status!=FLASH_COMPLETE?0:1);
}

void EraseFlash256(uint32_t address)
{
	if(address>FLASH_STORAGE_SIZE)
	{
		USBAdd8(0);
		return;
	}
	address+=FLASH_START_ARRAY;
	FLASH_Unlock();
	USBAdd8(FLASH_ErasePage(address)==FLASH_COMPLETE?1:0);
	FLASH_Lock();
}

void SendRtc(void)
{
	RTC_TimeTypeDef time;
	RTC_DateTypeDef date;
	RTC_GetTime(RTC_Format_BIN, &time);
	RTC_GetDate(RTC_Format_BIN, &date);

	USBAdd8(time.RTC_Seconds);
	USBAdd8(time.RTC_Minutes);
	USBAdd8(time.RTC_Hours);

	USBAdd8(date.RTC_Date);
	USBAdd8(date.RTC_Month);
	USBAdd8(date.RTC_Year);

}

void SetRtc(uint8_t* data)
{
	uint8_t ok = 0;
	RTC_TimeTypeDef time;
	RTC_DateTypeDef date;

	PWR_RTCAccessCmd(ENABLE);

	time.RTC_Seconds = data[0];
	time.RTC_Minutes = data[1];
	time.RTC_Hours = data[2];
	if(RTC_SetTime(RTC_Format_BIN, &time)!=SUCCESS)
		goto end;

	date.RTC_Date = data[3];
	date.RTC_Month = data[4];
	date.RTC_Year = data[5];
	date.RTC_WeekDay = RTC_Weekday_Monday;
	if(RTC_SetDate(RTC_Format_BIN, &date)!=SUCCESS)
		goto end;

	ok = 1;
end:
	USBAdd8(ok);

	PWR_RTCAccessCmd(DISABLE);
}

void ReadEepromByte(uint32_t address)
{
	if(address>=EEPROM_SIZE)
		return;
	address+=EEPROM_START_ADDRESS;

	USBAdd8(*(uint8_t*)address);
}

void WriteEepromByte(uint32_t address, uint8_t data)
{
	if(address>=EEPROM_SIZE)
	{
		USBAdd8(0);
		return;
	}

	address+=EEPROM_START_ADDRESS;
	DATA_EEPROM_Unlock();
	USBAdd8(DATA_EEPROM_FastProgramByte(address, data)==FLASH_COMPLETE?1:0);
	DATA_EEPROM_Lock();
}

void USBCommandReceive(uint8_t* commandBuffer, uint16_t commandSize)
{
	if(commandSize==0)
		return;
	switch(commandBuffer[0])
	{
	case 1://COMMAND_READ_EEPROM
		ReadEepromByte(*(uint32_t*)(commandBuffer+1));
		break;
	case 2://COMMAND_WRITE_EEPROM
		WriteEepromByte(*(uint32_t*)(commandBuffer+1), commandBuffer[5]);
		break;
	case 3://READ_FLASH_DATA32
		ReadFlashData32(*(uint32_t*)(commandBuffer+1));
		break;
	case 4://WRITE_FLASH_DATA32
		WriteFlashData32(*(uint32_t*)(commandBuffer+1), commandBuffer+5);
		break;
	case 5://ERASE_FLASH_DATA
		EraseFlash256(*(uint32_t*)(commandBuffer+1));
		break;
	case 7://COMMAND_CLEAR_LCD
		LCD_GLASS_Clear();
		break;
	case 8://COMMAND_SET_REGISTER
		LCD_WriteRegister(commandBuffer[1], *(uint32_t*)(commandBuffer+2));
		break;
	case 9://COMMAND_LCD_HEX
		while( LCD_GetFlagStatus (LCD_FLAG_UDR) != RESET) ;
		LCD_WriteHex(commandBuffer[1], *(uint16_t*)(commandBuffer+2));
		LCD_UpdateDisplayRequest();
		break;
	case 10://COMMAND_GET_RTC
		SendRtc();
		break;
	case 11://COMMAND_SET_RTC
		SetRtc(commandBuffer+1);
		break;
	case 12://COMMAND_FLASH_STORAGE_SIZE
		USBAdd32(FLASH_STORAGE_SIZE);
		break;
	}
	USBSend();	
}
