// balmer@inbox.ru 2013 TermometerHygrometer
#include "dht.h"

#define DHT_BITS_COUNT 41
extern uint16_t data[DHT_BITS_COUNT];
extern RCC_ClocksTypeDef RCC_Clocks;

static uint16_t bits_data[DHT_BITS_COUNT];

bool dht_ok = false;
bool dht_complete = false;
int16_t dht_temperature = 0;
int16_t dht_humidity = 0;

void DHTInit(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM6, ENABLE);

	GPIO_InitStructure.GPIO_Pin = DHT_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_Init(DHT_GPIO, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = DHT_POWER_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_Init(DHT_GPIO, &GPIO_InitStructure);

	GPIO_SetBits(DHT_GPIO, DHT_PIN);
	GPIO_SetBits(DHT_GPIO, DHT_POWER_PIN); //power off
}

void DHTPowerOn(bool on)
{
	if(on)
		GPIO_ResetBits(DHT_GPIO, DHT_POWER_PIN);
	else
		GPIO_SetBits(DHT_GPIO, DHT_POWER_PIN);
}

void DHTDelay5us(uint16_t count5us)
{
	uint16_t start = TIM6->CNT;

	while(TIM6->CNT-start < count5us);
}

void DHTDelay1ms(uint16_t count1ms)
{
	TIM_TimeBaseInitTypeDef tim_init;

	tim_init.TIM_Prescaler = RCC_Clocks.PCLK1_Frequency/1000-1; //1 ms per tick
	tim_init.TIM_CounterMode = TIM_CounterMode_Up;
	tim_init.TIM_Period = 0xFFFF;
	tim_init.TIM_ClockDivision = TIM_CKD_DIV1;

	TIM_SetCounter(TIM6, 0);
	TIM_TimeBaseInit(TIM6, &tim_init);
	TIM_Cmd(TIM6, ENABLE);

	while(TIM6->CNT < count1ms);

	TIM_Cmd(TIM6, DISABLE);
}
/*
	Включаем питание
	Ждем секунду
	Делаем low на 500 us
	Делаем high и начинаем смотреть когда переключилось на low и так 40 бит
*/
void DHTReadData(void)
{
	int i, j;
	#define  MAX_COUNT 30000
	int16_t temp;
	uint8_t data[5];

	dht_ok = false;
	dht_complete = false;
	//Timer6 у нас на APB1 = PCLK1_Frequency = 32000000
	TIM_TimeBaseInitTypeDef tim_init;
	GPIO_InitTypeDef GPIO_InitStructure;

	TIM_Cmd(TIM6, DISABLE);
	TIM_DeInit(TIM6);

	GPIO_InitStructure.GPIO_Pin = DHT_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_Init(DHT_GPIO, &GPIO_InitStructure);

	tim_init.TIM_Prescaler = RCC_Clocks.PCLK1_Frequency/200000-1; //5 us per tick
	tim_init.TIM_CounterMode = TIM_CounterMode_Up;
	tim_init.TIM_Period = 0xFFFF;
	tim_init.TIM_ClockDivision = TIM_CKD_DIV1;

	TIM_TimeBaseInit(TIM6, &tim_init);
	TIM_SetCounter(TIM6, 0);
	TIM_Cmd(TIM6, ENABLE);

	GPIO_ResetBits(DHT_GPIO, DHT_PIN);
	DHTDelay5us(200);//Delay 1000 us
	GPIO_SetBits(DHT_GPIO, DHT_PIN);

	GPIO_InitStructure.GPIO_Pin = DHT_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_Init(DHT_GPIO, &GPIO_InitStructure);
	//DHTDelay5us(8);//Delay 40 us
	//DHTDelay5us(4);


	for(i=0; i<DHT_BITS_COUNT; i++)
	{
		while( !(DHT_GPIO->IDR & DHT_PIN) && TIM6->CNT<MAX_COUNT );
		while( (DHT_GPIO->IDR & DHT_PIN) && TIM6->CNT<MAX_COUNT );
		bits_data[i] = TIM6->CNT;
	}

	TIM_Cmd(TIM6, DISABLE);

	for(i=0; i<5; i++)
	{
		uint8_t b = 0;
		for(j=0; j<8; j++)
		{
			int pos = i*8+j;
			int dt = bits_data[pos+1]-bits_data[pos];
			b <<= 1;
			if(dt>20)
				b|=1;
		}

		data[i] = b;
	}


	dht_ok =  (data[4] == ((data[0] + data[1] + data[2] + data[3]) & 0xFF));
	//temperature
	temp = data[2] & 0x7F;
	temp *= 256;
	temp += data[3];
	if (data[2] & 0x80)
		temp = -temp;

	dht_temperature = temp;

	//humidity
	dht_humidity = data[0]*256+data[1];
	dht_complete = true;
}
