// balmer@inbox.ru 2013 TermometerHygrometer

/* Includes ------------------------------------------------------------------*/
#include "ti8077_lcd.h"
#include "stm32l1xx_lcd.h"

/*
MY LCD 
COM0 - PA8
COM1 - PA9
COM2 - PA10

S0 - PA1 - PS23
S1 - PA2 - PS22
S2 - PA3 - PS21
S3 - PA6 - PS20
S4 - PA7 - PS19
S5 - PB0 - PS18
S6 - PB1 - PS17
S7 - PB3 - PS30
S8 - PB4 - PS29
S9 - PB5 - PS28
S10 - PB10 - PS16
S11 - PB11 - PS15
S12 - PB12 - PS14
S13 - PB13 - PS13
S14 - PB14 - PS12
S15 - PB15 - PS11
S18 - PC0 - PS27
S19 - PC1 - PS26
S20 - PC2 - PS25
S21 - PC3 - PS24
S24 - PC6 - PS10
S25 - PC7 - PS9
S26 - PC8 - PS8
S27 - PC9 - PS7

A - 1 2 3 6 7 8 9 10
B - 0 1 3 4 5 10 11 12 13 14 15
C - 0 1 2 3 6 7 8 9

  0000
 2    1
 2    1
 2    1
  3333
 5    4
 5    4
 5    4
  6666
       7
  8  8
   88
*/

static const uint32_t pins[] =
{
  0x4000000,//00
  0x8000000,//01
  0x2000000,//02
     0x8000,//10
  0x1000000,//11
     0x4000,//12
     0x1000,//20
     0x2000,//21
      0x800,//22
       0x40,//30
      0x400,//31
       0x20,//32
        0x8,//40
       0x10,//41
        0x4,//42
        0x1,//50
        0x2,//51
   0x200000,//52
    0x80000,//60
   0x100000,//61
    0x40000,//62
      0x100,//70
      0x200,//71
       0x80 //72
};

static const uint8_t digits[] =
{
    0x77,
    0x12,
    0x6B,
    0x5B,
    0x1E,
    0x5D,
    0x7D,
    0x13,
    0x7F,
    0x5F,
};

/**
  * @brief  Configures the LCD GLASS relative GPIO port IOs and LCD peripheral.
  * @param  None
  * @retval None
  */
void LCD_GLASS_Init(void)
{
  LCD_InitTypeDef LCD_InitStruct;

 
  LCD_InitStruct.LCD_Prescaler = LCD_Prescaler_1;
  LCD_InitStruct.LCD_Divider = LCD_Divider_31;
  LCD_InitStruct.LCD_Duty = LCD_Duty_1_3;
  LCD_InitStruct.LCD_Bias = LCD_Bias_1_3;
  LCD_InitStruct.LCD_VoltageSource = LCD_VoltageSource_Internal;

 
  /* Initialize the LCD */
  LCD_Init(&LCD_InitStruct);
  
  LCD_MuxSegmentCmd(DISABLE);
  
  /* To set contrast to mean value */
  LCD_ContrastConfig(LCD_Contrast_Level_4);
  
  LCD_DeadTimeConfig(LCD_DeadTime_0);
  LCD_PulseOnDurationConfig(LCD_PulseOnDuration_4);

  /* Wait Until the LCD FCR register is synchronized */
  LCD_WaitForSynchro();
  
  /* Enable LCD peripheral */
  LCD_Cmd(ENABLE);
  
  /* Wait Until the LCD is enabled */
  while(LCD_GetFlagStatus(LCD_FLAG_ENS) == RESET)
  {
  }
  /*!< Wait Until the LCD Booster is ready */  
  while(LCD_GetFlagStatus(LCD_FLAG_RDY) == RESET)
  {
  } 

  LCD_BlinkConfig(LCD_BlinkMode_Off,LCD_BlinkFrequency_Div32);	  
  LCD_GLASS_Clear();
}

/**
  * @brief  To initialize the LCD pins
  * @caller main
  * @param None
  * @retval None
  */

void LCD_GLASS_Configure_GPIO(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  
/* Enable GPIOs clock */ 	
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB | RCC_AHBPeriph_GPIOC, ENABLE);

/* Configure Output for LCD */
/* Port A */
  GPIO_StructInit(&GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_6 | GPIO_Pin_7 |GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_Init( GPIOA, &GPIO_InitStructure);

  GPIO_PinAFConfig(GPIOA, GPIO_PinSource1,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource2,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource3,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource6,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource7,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource8,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource9,GPIO_AF_LCD) ;  
  GPIO_PinAFConfig(GPIOA, GPIO_PinSource10,GPIO_AF_LCD) ;  
  
/* Configure Output for LCD */
/* Port B */  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5
                                 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;  
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_Init( GPIOB, &GPIO_InitStructure);
  
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource0,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource1,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource3,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource4,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource5,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource10,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource11,GPIO_AF_LCD) ;  
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource12,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource13,GPIO_AF_LCD) ;   
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource14,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOB, GPIO_PinSource15,GPIO_AF_LCD) ;   
  
/* Configure Output for LCD */
/* Port C*/  

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_6
                                 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;                
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_Init( GPIOC, &GPIO_InitStructure);  
  

  GPIO_PinAFConfig(GPIOC, GPIO_PinSource0,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource1,GPIO_AF_LCD) ; 
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource2,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource3,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource6,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource7,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource8,GPIO_AF_LCD) ;
  GPIO_PinAFConfig(GPIOC, GPIO_PinSource9,GPIO_AF_LCD) ;

/* Disable GPIOs clock */ 	
  RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA | RCC_AHBPeriph_GPIOB | RCC_AHBPeriph_GPIOC, DISABLE);
  
}

static uint16_t LCD_Conv_Char_Seg(uint8_t c,bool point,bool column)
{
  uint16_t ch = 0 ;

  if(c>='0' && c<='9')
  {
    ch = digits[c-'0'];
  } else
  switch (c)
  {
    case 'H':
      ch = 0x3E;
      break;
    case 'C':
      ch = 0x65;
      break;
    case '@':
      ch = 0x0F;//CELSIUS
      break;
    case '-':
      ch = 0x08;
      break;
    case 'U':
      ch = 0x76;
      break;
    default:
      ch = 0x00;
      break;
  }
       
  if (point)
    ch |= 0x80;

  if (column)
    ch |= 0x100;

  return ch;
}

void LCD_Test()
{
  while( LCD_GetFlagStatus (LCD_FLAG_UDR) != RESET) ;

  LCD->RAM[LCD_RAMRegister_0] |= 0x2;
  LCD->RAM[LCD_RAMRegister_2] |= 0x2;
  LCD->RAM[LCD_RAMRegister_4] |= 0x2;
  LCD_UpdateDisplayRequest();
}

void LCD_WriteRegister(uint8_t index, uint32_t value)
{
  while( LCD_GetFlagStatus (LCD_FLAG_UDR) != RESET) ;
  LCD->RAM[index] = value;
  LCD_UpdateDisplayRequest();
}

/*
Write 9 segments
*/
void LCD_WriteHex(uint8_t position, uint16_t hex9)
{
  int i,j;
  const uint32_t* bits;
  uint32_t bit;
  if(position>=8)
    return;
  bits = pins+position*3;

  for(j=0;j<3;j++)
  for(i=0;i<3;i++)
  {
    bit = bits[i];
    if(hex9&1)
      LCD->RAM[j*2] |= bit;
    else
      LCD->RAM[j*2] &= ~bit;
    hex9>>=1;
  }
}


void LCD_GLASS_WriteChar(uint8_t ch, bool point, bool column, uint8_t position)
{
  uint16_t hex9 = LCD_Conv_Char_Seg(ch,point,column);

  LCD_WriteHex(position, hex9);
}

void LCD_GLASS_DisplayString(char* ptr)
{
  int8_t i = 7;

	// wait for LCD Ready
  while( LCD_GetFlagStatus (LCD_FLAG_UDR) != RESET) ;
	
  while ((*ptr != 0) & (i >=0))
  {
    LCD_GLASS_WriteChar(*ptr, false, false, i);
    ptr++;
    i--;
  }
}

void LCD_GLASS_Clear(void)
{
  uint32_t counter = 0;
	
  /* TO wait LCD Ready */  
  while( LCD_GetFlagStatus (LCD_FLAG_UDR) != RESET) ;
  
  for (counter = LCD_RAMRegister_0; counter <= LCD_RAMRegister_15; counter++)
  {
    LCD->RAM[counter] = 0;
  }
  
}
