# -*- coding: UTF-8 -*-
# balmer@inbox.ru 2013 Termometer Hygrometer
import sys, os, csv
from PyQt4 import QtCore, QtGui

import matplotlib
import time
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.figure import Figure


class FormDrawData(QtGui.QMainWindow):
    def __init__(self, parent=None):
        super(FormDrawData, self).__init__(parent)
        self.setWindowTitle('Termomerer-hygrometer data')

        self.createMainFrame()
        pass        
    
    
    def createMainFrame(self):
        self.main_frame = QtGui.QWidget()
        
        plot_frame = QtGui.QWidget()
        
        self.dpi = 100
        self.fig = Figure((6.0, 4.0), dpi=self.dpi)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        
        self.axes = self.fig.add_subplot(111)
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)
        
        left_vbox = QtGui.QVBoxLayout()
        left_vbox.addWidget(self.canvas)
        left_vbox.addWidget(self.mpl_toolbar)

        right_vbox = QtGui.QVBoxLayout()
        right_vbox.addStretch(1)
        
        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(left_vbox, stretch=1)
        hbox.addLayout(right_vbox)
        self.main_frame.setLayout(hbox)

        self.setCentralWidget(self.main_frame)

    def setData(self, data):
		self.data = data
		self.updateFigure()

    def updateFigure(self):
		print "Show"
		self.axes.clear()
		self.axes.grid(True)

		xlist = self.data["dates"]

		print "date min=", xlist[0]
		print "date max=", xlist[-1]

		if "min_temperature" in self.data:
			self.axes.xaxis.set_major_locator( matplotlib.dates.DayLocator() )
			self.axes.xaxis.set_major_formatter( matplotlib.dates.DateFormatter('%Y-%m-%d') )
			self.axes.xaxis.set_minor_locator( matplotlib.dates.HourLocator() )
		else:
			self.axes.set_title(xlist[0].strftime('%Y-%m-%d'))
			self.axes.xaxis.set_major_locator( matplotlib.dates.HourLocator(interval=2) )
			self.axes.xaxis.set_major_formatter( matplotlib.dates.DateFormatter('%Hh') )
		self.axes.xaxis.grid(True, which="major")

		b_draw_T = True
		b_draw_H = True

		if b_draw_T:
			ax1 = self.axes

			ax1.plot(xlist, self.data["temperature"], 'r')
			if "min_temperature" in self.data:
				ax1.plot(xlist, self.data["min_temperature"], 'r--')
			if "max_temperature" in self.data:
				ax1.plot(xlist, self.data["max_temperature"], 'r--')

			ax1.set_ylabel('Celsius')
			for tl in ax1.get_yticklabels():
				tl.set_color('r')

		if b_draw_H:
			if b_draw_T:
				ax2 = self.axes.twinx()
			else:
				ax2 = self.axes
			ax2.plot(xlist, self.data["humidity"], 'b')
			if "min_humidity" in self.data:
				ax2.plot(xlist, self.data["min_humidity"], 'b--')
			if "max_humidity" in self.data:
				ax2.plot(xlist, self.data["max_humidity"], 'b--')

			ax2.set_ylabel('Humidity %')
			for tl in ax2.get_yticklabels():
				tl.set_color('b')	

		self.fig.autofmt_xdate()

		pass
