# -*- coding: UTF-8 -*-
# balmer@inbox.ru 2013 Termometer Hygrometer

import math
import datetime
import struct
import os

def readFileAsArray(filename):
	with open(filename, "rb") as file:
		data = file.read()
	return bytearray(data)

def convert256ToReadable(data, index):
	"""
	На входе 256 байт, прочитанных из термометра-гигрометра
	На выходе - время когда сэмплы начали считываться и 60 сэмплов 
	температуры и влажности.
    Input format
    uint8_t hours_count; // ==6
    uint8_t min;      // 0 to 59
    uint8_t hour;     // 0 to 23
    uint8_t mday;     // 1 to 31
    uint8_t mon;      // 1 to 12
    uint8_t year;     // year-99
    uint8_t padder0;  //0
    uint8_t padder1;  //0

    while
        2 byte temperature*10
        2 byte humidity%*10

	"""
	out = {}
	step_minutes = data[0]
	if not (step_minutes==6 or step_minutes==1):
		print "Error index="+str(index)
		print "data[0]="+str(data[0])
		return False
	
	out["date"] = datetime.datetime(year=data[5]+2000, month=data[4], day=data[3], hour=data[2], minute=data[1], second=0)
	#print "date=", out["date"]
	temperature = []
	humidity = []
	dates = []
	curDate = out["date"]
	for sample in xrange(0, 60):
		offset = sample*4+8
		t = struct.unpack_from("h", data, offset)[0]
		h = struct.unpack_from("h", data, offset+2)[0]
		if not (t==0 and h==0):
			t *= 0.1
			h *= 0.1
			temperature.append(t)
			humidity.append(h)
			dates.append(curDate)

		curDate += datetime.timedelta(minutes=step_minutes)

	out["temperature"] = temperature
	out["humidity"] = humidity
	out["dates"] = dates
	return out

def mergeMultiple(allData, beginIndex=0, endIndex=-1):
	if endIndex==-1:
		endIndex = len(allData)//256
	sum = None
	for i in xrange(beginIndex, endIndex):
		out = convert256ToReadable(allData[i*256:i*256+256], i)
		if not out:
			continue
		if not sum:
			sum = out
			continue

		sum["temperature"] += out["temperature"]
		sum["humidity"] += out["humidity"]
		sum["dates"] += out["dates"]
	return sum

def mergeMultipleByDate(allData, beginDate=None, endDate=None):
	beginIndex = 0
	endIndex = len(allData)//256
	sum = None
	for i in xrange(beginIndex, endIndex):
		out = convert256ToReadable(allData[i*256:i*256+256], i)
		if not out:
			continue

		if beginDate:
			if out["dates"][0]<beginDate:
				continue
		if endDate:
			if out["dates"][-1]>endDate:
				break

		if not sum:
			sum = out
			continue

		sum["temperature"] += out["temperature"]
		sum["humidity"] += out["humidity"]
		sum["dates"] += out["dates"]
	return sum

def readThFile(filename):
	data = readFileAsArray(filename)
	return mergeMultiple(data)

def readLastDate(filename):
	try:
		with open(filename, "rb") as file:
			file.seek(0, os.SEEK_END)
			size = file.tell()
			if size<256:
				return None
			lastPage = ((size-256)//256)*256
			file.seek(lastPage, os.SEEK_SET)
			data = bytearray(file.read(256))
			rd = convert256ToReadable(data, 0)
	except IOError:
		return None

	return rd["date"]
