# coding=UTF-8
import usb
import time
import datetime
import os
import struct
import sys
import read

dev = None
log_text_edit = None

def log(text):
    global log_text_edit
    if log_text_edit:
        log_text_edit.append(text)
    else:
        print text
    pass

COMMAND_READ_EEPROM = 1
COMMAND_WRITE_EEPROM = 2
COMMAND_READ_FLASH_DATA32 = 3
COMMAND_WRITE_FLASH_DATA32 = 4
COMMAND_ERASE_FLASH_DATA = 5
COMMAND_CLEAR_LCD = 7
COMMAND_SET_REGISTER = 8
COMMAND_LCD_HEX = 9
COMMAND_GET_RTC = 10
COMMAND_SET_RTC = 11
COMMAND_FLASH_STORAGE_SIZE = 12


def findDevice():
    global dev
    dev = usb.core.find(idVendor=0x16C0, idProduct=0x05DC)

    if dev is None:
        log('Device not found')
        return False
    else:
        log("Device found")

    dev.set_configuration()
    return True

def readAll():
    time.sleep(0.1)
    while True:
        try:
            data = dev.read(129, 128, interface=1, timeout=50)
            if len(data)==0:
                break
            print "readb=", data
            print "read=", data.tostring()
        except usb.core.USBError:
            break
    pass

def readOne():
    time.sleep(0.1)
    try:
        data = dev.read(129, 128, interface=1, timeout=50)
        if len(data)==0:
            print "Read empty"
        else:
            print "readb=", data
            #print "read=", data.tostring()
    except usb.core.USBError:
        print "Read USB error"
    pass

def readInt():
    data = dev.read(129, 128, interface=1, timeout=50)
    out = None
    if len(data)!=4:
        log("Fail read int len="+str(len(data)))
    else:
        out = data[0]+data[1]*0x100+data[2]*0x10000+data[3]*0x1000000
    return out

def printEndpoint(e):
    print "Endpoint:"
    print "bLength=", e.bLength
    print "bDescriptorType=", e.bDescriptorType
    print "bEndpointAddress=", e.bEndpointAddress
    print "bmAttributes=", e.bmAttributes
    print "wMaxPacketSize=", e.wMaxPacketSize
    print "bInterval=", e.bInterval
    print "bRefresh=", e.bRefresh
    print "bSynchAddress=", e.bSynchAddress
    pass

def uint32ToBytes(address):
    return [address%0x100, (address//0x100)%0x100, (address//0x10000)%0x100, (address//0x1000000)%0x100]

def readFlash(address):
    '''
    read 32 byte flash data
    '''
    dev.write(3, [COMMAND_READ_FLASH_DATA32]+uint32ToBytes(address), interface=1)
    readOne()
    pass

def eraseFlash(address):
    ''' Erase 256 bytes flash data
    '''
    print "eraseFlash"
    dev.write(3, [COMMAND_ERASE_FLASH_DATA]+uint32ToBytes(address), interface=1)
    readOne()
    pass

def writeFlash(address, array):
    ''' Write 32 byte flash data
    '''
    dev.write(3, [COMMAND_WRITE_FLASH_DATA32]+uint32ToBytes(address)+array, interface=1)
    readOne()
    pass

def readEepromByte(address):
    dev.write(3, [COMMAND_READ_EEPROM]+uint32ToBytes(address), interface=1)
    data = dev.read(129, 128, interface=1, timeout=50)
    return data[0]

def writeEepromByte(address, byte):
    dev.write(3, [COMMAND_WRITE_EEPROM]+uint32ToBytes(address)+[byte], interface=1)
    readOne()
    pass

def lcdClear():
    dev.write(3, [COMMAND_CLEAR_LCD], interface=1)
    #readOne()

def lcdSet(offset, value):
    command = [COMMAND_SET_REGISTER, offset, value%0x100, (value/0x100)%0x100, (value/0x10000)%0x100, (value/0x1000000)%0x100]
    print command
    dev.write(3, command, interface=1)
    #readOne()

def lcdSet3(value):
    lcdSet(0, value)
    lcdSet(2, value)
    lcdSet(4, value)

def lcdHex(offset, value):
    command = [COMMAND_LCD_HEX, offset, value%0x100, (value/0x100)%0x100]
    print command
    dev.write(3, command, interface=1)

def getDateAndTime():
    dev.write(3, [COMMAND_GET_RTC], interface=1)
    data = dev.read(129, 128, interface=1, timeout=50)
    if len(data)==0:
        return None
    dt = datetime.datetime(year = data[5]+2000, month = data[4], day = data[3], hour= data[2], minute = data[1], second = data[0]);
    log("date="+str(dt))
    return dt

def setCurDateAndTime():
    cur = datetime.datetime.today()
    log("cur date="+str(cur))
    #print [COMMAND_SET_RTC, cur.second, cur.minute, cur.hour, cur.day, cur.month, cur.year-2000]
    dev.write(3, [COMMAND_SET_RTC, cur.second, cur.minute, cur.hour, cur.day, cur.month, cur.year-2000], interface=1)
    readOne()
    pass

def setCurDateAndTimeMidnight():
    cur = datetime.datetime.today()
    dev.write(3, [COMMAND_SET_RTC, 40, 59, 23, cur.day, cur.month, cur.year-2000], interface=1)
    readOne()
    pass

def getFlashStorageSize():
    dev.write(3, [COMMAND_FLASH_STORAGE_SIZE], interface=1)
    return readInt()


def readFlash256(address):
    ret = bytearray(256)
    for offset in xrange(0, 256, 32):
        dev.write(3, [COMMAND_READ_FLASH_DATA32]+uint32ToBytes(address+offset), interface=1)
        data = dev.read(129, 128, interface=1, timeout=50)
        for i in xrange(32):
            ret[offset+i] = data[i]
    return ret
        
def setName(deviceName):
    index = 4
    for c in deviceName:
        writeEepromByte(index, ord(c))
        index += 1

    writeEepromByte(index, 0)
    pass

def getName():
    index = 4
    text = ''
    while(True):
        s = readEepromByte(index)
        if s<32 or s>=128:
            break
        text = text + chr(s)
        index += 1

    return text

def writeDataToFileOld(filename):
    with open(filename, "w+b") as file:
        file.seek(0, os.SEEK_END)
        size = file.tell()
        minAddress = (size//256)*256

        #last page may be not full
        #check underflow!!!!!!!!!
        if minAddress>=256:
            minAddress -= 256

        log("minAddress="+str(minAddress))
        file.seek(minAddress, os.SEEK_SET)

        maxAddress = readEepromByte(0) + readEepromByte(1)*0x100 + readEepromByte(2)*0x10000

        #check overflow!!!!!!!!
        maxAddress +=256
        log("maxAddress="+str(maxAddress))

        curAddress  = minAddress
        while curAddress<maxAddress:
            log("curPage="+str(curAddress//256))
            bytes = readFlash256(curAddress)
            file.write(bytes)
            curAddress += 256

def writeDataToFile(filename):
    flashSize = getFlashStorageSize()
    flashSizePages = flashSize//256
    minDate = read.readLastDate(filename)
    log("minFileDate = "+str(minDate))

    maxPage = readEepromByte(1) + readEepromByte(2)*0x100
    datas = []
    minDateFound = False 
    for i in xrange(0, flashSizePages):
        curPage = maxPage - i
        if curPage<0:
            curPage+=flashSizePages
        log("curPage="+str(curPage))
        bytes = readFlash256(curPage*256)
        if bytes[0]==0:
            break
        data = read.convert256ToReadable(bytes, 0)
        if data["date"]>=minDate:
            log("date="+str(data["date"]))
            datas.append(bytes)
            if data["date"]==minDate:
                minDateFound = True
                break
        else:
            break


    with open(filename, "a+b") as file:
        file.seek(0, os.SEEK_END)
        size = file.tell()
        minAddress = (size//256)*256

        if minAddress>=256 and minDateFound:
            minAddress -= 256

        log("minAddress="+str(minAddress))
        file.seek(minAddress, os.SEEK_SET)
        file.truncate()

        for d in reversed(datas):
            file.write(d)

def main(command, name_to_set=None):
    if not findDevice():
        return

    #readOne()
    getDateAndTime()
    getDateAndTime()

    name = getName()
    log('device name = '+name)
    log('storage size = '+str(getFlashStorageSize()))

    if command=='set_name':
        setName(name_to_set)
        log('new device name = '+name_to_set)
    elif command=='get_name':
        pass
    elif command=='read_data':
        writeDataToFile("data/th_"+name+".dat")
    elif command=='set_time':
        setCurDateAndTime()
        #setCurDateAndTimeMidnight()
        log('Set time complete')
    elif command=="addr":
        for i in xrange(0, 4):
            log("eeprom"+str(i)+"="+str(readEepromByte(i)))
    elif command=="clear_counter":
        for i in xrange(0, 4):
            writeEepromByte(i, 0)
        eraseFlash(0)
    else:
        print "Unknown command"
        pass

    log('Query complete.')

if __name__ == "__main__":
    name_to_set = None
    if sys.argv[1]=='set_name':
        name_to_set = sys.argv[2]

    main(sys.argv[1], name_to_set=name_to_set)

