# -*- coding: UTF-8 -*-
# balmer@inbox.ru 2013 Termometer Hygrometer
import sys, os, csv
from PyQt4 import QtCore, QtGui

import matplotlib
import time
import datetime
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.figure import Figure

from code.read import readThFile
from code.plot import FormDrawData
import code.usb_commands


class Form(QtGui.QMainWindow):

    def __init__(self, parent=None):
        super(Form, self).__init__(parent)

        self.communicate_thread = None

        self.setWindowTitle('Termometer-Hygrometer dashboard (Balmer 2013)')
        self.CreateMainFrame()
        pass        
        
    def CreateMainFrame(self):
        self.main_frame = QtGui.QWidget()

        header_label = QtGui.QLabel(
            u'Сервисная утилита для термометра-гигрометра на STM32.\n'
            )

        data_group = QtGui.QGroupBox(u'Просмотр данных')
        device_group = QtGui.QGroupBox(u'Общение с устройством')
      

        device_group.setLayout(self.CreateCommunicateFrame())
        data_group.setLayout(self.CreateDataFrame())
        
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(data_group)
        hbox.addWidget(device_group)

        vbox = QtGui.QVBoxLayout()
        vbox.addWidget(header_label)
        vbox.addLayout(hbox)

        self.main_frame.setLayout(vbox)

        self.setCentralWidget(self.main_frame)

    def CreateCommunicateFrame(self):
        ''' Return layout '''
        vbox = QtGui.QVBoxLayout()
        connect_button = QtGui.QPushButton(u'Прочитать имя датчика')
        connect_button.clicked.connect(self.OnCommunicateName)

        read_button = QtGui.QPushButton(u'Прочитать данные')
        read_button.clicked.connect(self.OnCommunicateRead)
        
        time_button = QtGui.QPushButton(u'Установить время')
        time_button.clicked.connect(self.OnCommunicateSetTime)

        self.log_text_edit = QtGui.QTextEdit()
        self.log_text_edit.setReadOnly(True)

        vbox.addWidget(connect_button)
        vbox.addWidget(read_button)
        vbox.addWidget(time_button)
        vbox.addWidget(self.log_text_edit, stretch=1)
        return vbox

    def CreateDataFrame(self):
        ''' Return layout '''
        vbox = QtGui.QVBoxLayout()

        info_label = QtGui.QLabel(u'Выбор датчика\n')

        sensor_list = getSensorList()

        label_sensor = QtGui.QLabel(u'Датчик')
        self.sensor_combo_box = QtGui.QComboBox()

        for name in sensor_list:
            self.sensor_combo_box.addItem(name, QtCore.QVariant('data/th_'+name+'.dat'))
        self.sensor_combo_box.currentIndexChanged.connect(self.OnSelectSensor)

        sensor_hbox = QtGui.QHBoxLayout()
        sensor_hbox.addWidget(label_sensor)
        sensor_hbox.addWidget(self.sensor_combo_box)
        sensor_hbox.addStretch(1)

        mindate_hbox = QtGui.QHBoxLayout()
        mindate_hbox.addWidget(QtGui.QLabel(u'Минимальная дата:'))
        self.mindate_label = QtGui.QLabel(u'00.00.0000')
        mindate_hbox.addWidget(self.mindate_label)

        maxdate_hbox = QtGui.QHBoxLayout()
        maxdate_hbox.addWidget(QtGui.QLabel(u'Максимальная дата:'))
        self.maxdate_label = QtGui.QLabel(u'00.00.0000')
        maxdate_hbox.addWidget(self.maxdate_label)

        curdate_hbox = QtGui.QHBoxLayout()
        curdate_hbox.addWidget(QtGui.QLabel(u'Дата начала графика:'))
        self.curdate_edit = QtGui.QDateEdit()
        curdate_hbox.addWidget(self.curdate_edit)
        
        self.days_spin = QtGui.QSpinBox()
        self.days_spin.setPrefix(u'Дней на обзорном графике:')
        self.days_spin.setRange(1, 120)
        self.days_spin.setValue(10)

        detailed_button = QtGui.QPushButton(u'Подробный график')
        detailed_button.clicked.connect(self.OnDetailedGraphButton)
        synoptic_button = QtGui.QPushButton(u'Обзорный график')
        synoptic_button.clicked.connect(self.OnSynopticGraphButton)

        vbox.addWidget(info_label)
        vbox.addLayout(sensor_hbox)
        vbox.addLayout(mindate_hbox)
        vbox.addLayout(maxdate_hbox)
        vbox.addLayout(curdate_hbox)
        vbox.addWidget(detailed_button)
        vbox.addWidget(self.days_spin)
        vbox.addWidget(synoptic_button)
        vbox.addStretch(1)

        if len(sensor_list)>0:
            self.OnSelectSensor(0)
        return vbox
    

    def OnSelectSensor(self, index):
        filename = self.sensor_combo_box.itemData(index).toString()
        print filename
        self.th_data = readThFile(filename)

        self.min_date = toQtDateTime(self.th_data["dates"][0]).date()
        self.mindate_label.setText(self.min_date.toString('yyyy.MM.dd'))

        self.max_date = toQtDateTime(self.th_data["dates"][-1]).date()
        self.maxdate_label.setText(self.max_date.toString('yyyy.MM.dd'))

        self.curdate_edit.setDateRange(self.min_date, self.max_date)
        self.curdate_edit.setDate(self.min_date)
        pass

    def OnDetailedGraphButton(self):
        startDate = self.curdate_edit.date()

        startDateTime = datetime.datetime(year=startDate.year(), month=startDate.month(), day=startDate.day())

        endDateTime = startDateTime + datetime.timedelta(days=1)

        dayData = extractDateRange(self.th_data, startDateTime, endDateTime)

        draw_data = FormDrawData(self)
        draw_data.setData(dayData)
        draw_data.show()
        pass

    def OnSynopticGraphButton(self):
        startDate = self.curdate_edit.date()

        startDateTime = datetime.datetime(year=startDate.year(), month=startDate.month(), day=startDate.day())

        endDateTime = startDateTime + datetime.timedelta(days=self.days_spin.value())

        dayData = extractDateRange(self.th_data, startDateTime, endDateTime)
        dayData = averageDataHour(dayData)

        draw_data = FormDrawData(self)
        draw_data.setData(dayData)
        draw_data.show()
        pass

    def OnCommunicateName(self):
        self.communicateCommand('get_name')
        pass

    def OnCommunicateRead(self):
        self.communicateCommand('read_data')
        pass

    def OnCommunicateSetTime(self):
        self.communicateCommand('set_time')
        pass

    def communicateCommand(self, command):
        if self.communicate_thread:
            if self.communicate_thread.isRunning():
                print "Already pressed!"
                return
        self.communicate_thread = CommunicateThread()
        self.log_text_edit.clear()
        code.usb_commands.log_text_edit = self.log_text_edit
        self.communicate_thread.command = command
        self.communicate_thread.start()


class CommunicateThread(QtCore.QThread):
    command = None

    def log(self, outString):
        self.log_text_edit.append(outString)
        pass

    def run(self):
        code.usb_commands.main(self.command)
        pass

def getSensorList():
    ''' В директории data лежат файлы вида
    th_name.dat
    name -  имя сенсора
    '''
    content = os.listdir('data')

    sensor_names = []
    for f in content:
        if f.find('th_')==0:
            if f.rfind('.dat')==len(f)-4:
                sensor_names.append(f[3:-4])

    return sensor_names

def toQtDateTime(pyDateTime):
    return QtCore.QDateTime(pyDateTime.year, pyDateTime.month, pyDateTime.day, pyDateTime.hour, pyDateTime.minute, pyDateTime.second)
    pass

def extractDateRange(data, dateMin, dateMax):
    indexMin = None
    indexMax = None
    temperature = data["temperature"]
    humidity = data["humidity"]
    dates = data["dates"]
    for i in xrange(0, len(dates)):
        if dates[i]>=dateMin:
            indexMin = i
            break

    if indexMin==None:
        indexMin = len(dates)-1

    for i in xrange(indexMin, len(dates)):
        if dates[i]>=dateMax:
            indexMax = i
            break

    if indexMax==None:
        indexMax = len(dates)-1

    out = {}
    out["temperature"] = temperature[indexMin:indexMax]
    out["humidity"] = humidity[indexMin:indexMax]
    out["dates"] = dates[indexMin:indexMax]
    #print out
    return out

def averageDataHour(data):
    '''
    Усредняем все данные по часам
    '''
    temperature = data["temperature"]
    humidity = data["humidity"]
    dates = data["dates"]

    out_temperature = []
    out_humidity = []
    out_dates = []
    out_min_temperature = []
    out_max_temperature = []
    out_min_humidity = []
    out_max_humidity = []

    cur_date = None
    max_date = None

    for index in xrange(0, len(dates)):
        t = temperature[index]
        h = humidity[index]

        if cur_date==None or cur_date.hour!=dates[index].hour:
            if cur_date!=None:
                out_temperature.append(sum_temperature/cur_count)
                out_humidity.append(sum_humidity/cur_count)
                out_dates.append(cur_date+(max_date-cur_date)/2)
                out_min_temperature.append(min_temperature)
                out_max_temperature.append(max_temperature)
                out_min_humidity.append(min_humidity)
                out_max_humidity.append(max_humidity)
                pass
            cur_count = 0
            cur_date = dates[index]
            sum_temperature = 0
            sum_humidity = 0
            min_temperature = t
            max_temperature = t
            min_humidity = h
            max_humidity = h

        max_date = dates[index]
        sum_temperature += t
        sum_humidity += h
        cur_count += 1

        if t<min_temperature:
            min_temperature = t
        if t>max_temperature:
            max_temperature = t
        if h<min_humidity:
            min_humidity = h
        if h>max_humidity:
            max_humidity = h
        pass

    if cur_date!=None:
        out_temperature.append(sum_temperature/cur_count)
        out_humidity.append(sum_humidity/cur_count)
        out_dates.append(cur_date+(max_date-cur_date)/2)
        out_min_temperature.append(min_temperature)
        out_max_temperature.append(max_temperature)
        out_min_humidity.append(min_humidity)
        out_max_humidity.append(max_humidity)

    out = {}
    out["temperature"] = out_temperature
    out["humidity"] = out_humidity
    out["dates"] = out_dates
    out["min_temperature"] = out_min_temperature
    out["max_temperature"] = out_max_temperature
    out["min_humidity"] = out_min_humidity
    out["max_humidity"] = out_max_humidity
    return out


def main():
    app = QtGui.QApplication(sys.argv)
    form = Form()
    form.show()
    app.exec_()


if __name__ == "__main__":
    main()
    
